<?php

class WC_PayoMatix_Payment_Gateway extends WC_Payment_Gateway
{
    private $order_status;

    public function __construct()
    {
        $this->id = "payomatix_payment";
        $this->method_title = __(
            "Payomatix Payment",
            "woocommerce-payomatix-gateway"
        );
        $this->method_description = __(
            "Payomatix Payment getway provide direct payment",
            "woocommerce-payomatix-gateway"
        );
        $this->title = __("Payomatix Payment", "woocommerce-payomatix-gateway");
        $this->has_fields = true;
        $this->init_form_fields();
        $this->init_settings();
        $this->enabled = $this->get_option("enabled");
        $this->title = $this->get_option("title");
        $this->description = $this->get_option("description");
        $this->hide_text_box = $this->get_option("hide_text_box");
        $this->text_box_required = $this->get_option("text_box_required");
        $this->order_status = $this->get_option("order_status");

        add_action("woocommerce_update_options_payment_gateways_" . $this->id, [
            $this,
            "process_admin_options",
        ]);
        
    }

    public function GetPaymentCategories(){

        $curl = curl_init();

        curl_setopt_array($curl, array(
        CURLOPT_URL => 'https://admin.payomatix.com/api/get-multi-connector-categories',
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => '',
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => 'GET',
        ));

        $response = curl_exec($curl);
        curl_close($curl);
        $categories = json_decode( $response, true );
        $categoriesData = [];
        if( isset($categories['data']) && !empty($categories['data']) ){
            foreach( $categories['data'] as $key => $value ){
                $categoriesData[] = $value['name'];
            }
        }
        return $categoriesData;
    }

    public function init_form_fields()
    {

        $categories = $this->GetPaymentCategories();

        $this->form_fields = [
            "enabled" => [
                "title" => __(
                    "Enable/Disable",
                    "woocommerce-payomatix-gateway"
                ),
                "type" => "checkbox",
                "label" => __(
                    "Enable Payomatix Payment",
                    "woocommerce-payomatix-gateway"
                ),
                "default" => "no",
            ],

            "title" => [
                "title" => __("Method Title", "woocommerce-payomatix-gateway"),
                "type" => "text",
                "description" => __(
                    "This controls the title",
                    "woocommerce-payomatix-gateway"
                ),
                "default" => __(
                    "Custom Payment",
                    "woocommerce-payomatix-gateway"
                ),
                "desc_tip" => true,
            ],
            "description" => [
                "title" => __(
                    "Customer Message",
                    "woocommerce-payomatix-gateway"
                ),
                "type" => "textarea",
                "css" => "width:500px;",
                "default" =>
                    "None of the other payment options are suitable for you? please drop us a note about your favourable payment option and we will contact you as soon as possible.",
                "description" => __(
                    "The message which you want it to appear to the customer in the checkout page.",
                    "woocommerce-payomatix-gateway"
                ),
            ],
            "testmode" => [
                "title" => __("TestMode", "woocommerce-payomatix-gateway"),
                "type" => "checkbox",
                "label" => __(
                    "TestMode Enable for test Payomatix Payment",
                    "woocommerce-payomatix-gateway"
                ),
                "default" => "yes",
            ],
            // "payment_button" => [
            //     "title" => __("Enable Payment Button", "woocommerce-payomatix-gateway"),
            //     "type" => "checkbox",
            //     "label" => __(
            //         "Enable Payment Button",
            //         "woocommerce-payomatix-gateway"
            //     ),
            //     "default" => "no",
            // ],
            "api_key" => [
                "title" => __("Secret Key", "woocommerce-payomatix-gateway"),
                "type" => "text",
                "description" => __("Api key", "woocommerce-payomatix-gateway"),
                "default" => __("Api Key", "woocommerce-payomatix-gateway"),
                "desc_tip" => true,
            ],
            "order_status" => [
                "title" => __(
                    "Order Status After The Checkout",
                    "woocommerce-payomatix-gateway"
                ),
                "type" => "select",
                "options" => wc_get_order_statuses(),
                "default" => "wc-on-hold",
                "description" => __(
                    "The default order status if this gateway used in payment.",
                    "woocommerce-payomatix-gateway"
                ),
            ],
            "payment_category_id" => [
                "title" => __(
                    "Select Payment Category",
                    "woocommerce-payomatix-gateway"
                ),
                "type" => "select",
                "options" => $categories,
                "default" => "Default",
                "description" => __(
                    "Default Payment category.",
                    "woocommerce-payomatix-gateway"
                ),
            ],
        ];
    }

    public function validate_fields()
    {
        if ($this->text_box_required === "no") {
            return true;
        }

        return true;
    }

    public function process_payment($order_id)
    {
        global $woocommerce;
        $order = new WC_Order($order_id);
        $items = $woocommerce->cart->get_cart();
        // $user_id = get_post_meta($order_id, "_customer_user", true);
        $user_id = $order->get_customer_id();
        $amount = (float) $order->get_total();
        $user_email = $order->get_billing_email(); // Get account email
        $currency = get_woocommerce_currency();
        $mobile = $order->get_billing_phone();
        $billing_first_name = $order->get_billing_first_name();
        $billing_last_name = $order->get_billing_last_name();
        $billing_company = $order->get_billing_company();
        $billing_address_1 = $order->get_billing_address_1();
        $billing_address_2 = $order->get_billing_address_2();
        $billing_city = $order->get_billing_city();
        $billing_state = $order->get_billing_state();
        $billing_postcode = $order->get_billing_postcode();
        $billing_country = $order->get_billing_country();

        $ip = $_SERVER["REMOTE_ADDR"];
        $apikey = $this->settings["api_key"];
        $mode = $this->settings["testmode"];
        if ($mode == "yes") {
            $url = "https://admin.payomatix.com/payment/merchant/transaction";
        } elseif ($mode == "no") {
            $url = "https://admin.payomatix.com/payment/merchant/transaction";
        }

        $products = [];
        foreach($items as $item => $values) {
            $_product =  wc_get_product( $values['data']->get_id() );
            $products[ $values['data']->get_id() ]['product_id'] = $values['data']->get_id();
            $products[ $values['data']->get_id() ]['name'] = $_product->get_title();
            $products[ $values['data']->get_id() ]['quantity'] = $values['quantity'];
            $products[ $values['data']->get_id() ]['price'] = get_post_meta($values['product_id'] , '_price', true);
            $products[ $values['data']->get_id() ]['image_url'] = wp_get_attachment_url( $_product->get_image_id() );
            $products[ $values['data']->get_id() ]['description'] = $_product->get_short_description();
            $products[ $values['data']->get_id() ]['product_code'] = $_product->get_sku();
        }

        $args = [
            "search_key" => $this->settings["payment_category_id"],
            "other_data" => [
                'first_name' => $_POST['billing_first_name'],
                'last_name' => $_POST['billing_last_name'],
                'address' => $_POST['billing_address_1'].''.$_POST['billing_address_2'],
                'country' => $_POST['billing_country'],
                'state' => $_POST['billing_state'],
                'city' => $_POST['billing_city'],
                'zip' => $_POST['billing_postcode'],
                'phone_no' => $_POST['billing_phone'],
                "products" => $products,
            ],
            "email" => $_POST["billing_email"],
            "amount" => sprintf("%0.2f", $amount),
            "currency" => $currency,
            // "card_no" => $_POST["card"]["cardno"],
            "merchant_ref" => $order_id,
            // "return_url" => site_url("payomatix-callback"),
            "return_url" => admin_url("admin-ajax.php")."?action=payomatix_callback_pending_tx",
            "notify_url" => admin_url("admin-ajax.php")."?action=payomatix_update_pending_tx&ostatus=".$this->order_status,
        ];

        $orrderStatus = $order->get_status();
        if ($orrderStatus == "failed") {
            $args["email"] = $order->get_billing_email();
        }

        $curl = curl_init();
        $postData = json_encode($args);
        curl_setopt_array($curl, [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_SSL_VERIFYHOST => false,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => $postData,
            CURLOPT_HTTPHEADER => ["Content-Type: application/json", "Authorization: ".$apikey],
        ]);

        $response = curl_exec($curl);
        curl_close($curl);
        $result = json_decode($response, true);
        
        if (isset($result["status"]) && $result["status"] == "success") {
            // Mark as on-hold (we're awaiting the cheque)
            $order->update_status(
                $this->order_status,
                __("Awaiting payment", "woocommerce-payomatix-gateway")
            );
            // Reduce stock levels
            wc_reduce_stock_levels($order_id);
            add_action('woocommerce_before_thankyou', 'custome_message_payment_payomatix_success');

            $order->add_order_note(
                esc_html("payment_order_id : " . $result["data"]["order_id"]),
                false
            );

            // Remove cart
            $woocommerce->cart->empty_cart();
            // Return thankyou redirect
            return [
                "result" => "success",
                "order_no" => $result["data"]["order_id"],
                "redirect" => $this->get_return_url($order),
            ];
        } elseif (
            isset($result["status"]) &&
            $result["status"] == "redirect"
        ) {
            $order->update_status(
                "wc-pending",
                __("Awaiting payment", "woocommerce-payomatix-gateway")
            );
            $order->add_order_note(
                esc_html(
                    "Order goes to the 3ds redirect : " .
                        $result["redirect_url"]
                ),
                false
            );

            // Remove cart
            $woocommerce->cart->empty_cart();

            return [
                "result" => "success",
                "redirect" => $result["redirect_url"],
            ];
        } else {
            $messageError = "Error : ";
            if (isset($result["errors"]['validation']) && !empty($result["errors"]['validation'])) {
                foreach ($result["errors"]['validation'] as $error) {
                    $messageError .= $error . ",";
                }
                $messageError = rtrim($messageError, ",");
                
                wc_add_notice(
                    __($messageError, "woocommerce-payomatix-gateway"),
                    "error"
                );
            } else {
                $messageError = $result["responseMessage"];
                if( isset($result['message']) && !empty($result['message']) ){
                    $messageError .= $result['message'];
                }
                wc_add_notice(
                    __($messageError, "woocommerce-payomatix-gateway"),
                    "error"
                );
            }
            return false;
        }
    }

    public function payment_fields()
    {
        ?>
		<fieldset>
			
            <p>
                You will be redirect to bank page for payment.
            </p>

		</fieldset>
		<?php
    }
}

function PayoMatixupdatePendingTranscations()
{
    $rawdata = file_get_contents("php://input");
    $orderDetails = json_decode($rawdata, true);
    
    $orderId = $orderDetails["customer_order_id"];
    $ostatus = $_REQUEST["ostatus"];
    $status = $orderDetails["transaction_status"];
    $message = isset($orderDetails["reason"]) ? $orderDetails["reason"] : "";
    if (empty($message)) {
        $message = isset($orderDetails["message"])
            ? $orderDetails["message"]
            : "";
    }

    global $woocommerce;

    // we need it to get any order detailes
    $order = wc_get_order($orderId);
    if( !empty($message) ){
        $order->add_order_note($message, false);
    }

    if ($status == "success") {
        // we need it to get any order detailes

        $order->payment_complete();
        $order->reduce_order_stock();
        if( !empty($message) ){
            $order->add_order_note($message, false);
        }
        $order->update_status($ostatus, $message);
        $woocommerce->cart->empty_cart();
        wc_add_notice($message, "Success");
        wc_add_notice(__($message, "woocommerce"), "success");
    } elseif ($status == "pending") {
        $order->payment_complete();
        if( !empty($message) ){
            $order->add_order_note($message, false);
        }
        $order->update_status("wc-failed", $message);
        $woocommerce->cart->empty_cart();
        wc_add_notice($message, "Error");
        wc_add_notice(__($message, "woocommerce"), "error");
    } else {
        if( !empty($message) ){
            $order->add_order_note($message, false);
        }
        $order->update_status("wc-failed", $message);
        $woocommerce->cart->empty_cart();
        wc_add_notice($message, "Error");
        wc_add_notice(__($message, "woocommerce"), "error");
    }

    wp_die();
}

add_action("wp_ajax_nopriv_payomatix_update_pending_tx", "PayoMatixupdatePendingTranscations");
add_action("wp_ajax_payomatix_update_pending_tx","PayoMatixupdatePendingTranscations");


function PayomatixCallbackPendingTransaction(){

    $orderId = $_REQUEST["merchant_ref"];
    $status = $_REQUEST["status"];
    $message = isset($_REQUEST["reason"])
        ? $_REQUEST["reason"]
        : "";
    if (empty($message)) {
        $message = isset($_REQUEST["message"])
            ? $_REQUEST["message"]
            : "";
    }

    global $woocommerce;
    $order = new WC_Order($orderId); 
    
    $payment_gateway_id = 'payomatix_payment';
    $payment_gateways   = WC_Payment_Gateways::instance();
    $payment_gateway    = $payment_gateways->payment_gateways()[$payment_gateway_id];


    if ($status == "success") {
        $order->payment_complete();
        $order->reduce_order_stock();
        if( !empty($message) ){
            $order->add_order_note($message, false);
        }
        $order->update_status($payment_gateway->settings['order_status'], $message);
        $woocommerce->cart->empty_cart();
        wc_add_notice($message, "Success");
        wc_add_notice(__($message, "woocommerce"), "success");
        $order_url = $payment_gateway->get_return_url($order);
        wp_redirect($order_url);
        exit();
    } elseif ($status == "declined") {
        $order->payment_complete();
        if( !empty($message) ){
            $order->add_order_note($message, false);
        }
        $order->update_status("wc-failed", $message);
        $woocommerce->cart->empty_cart();
        wc_add_notice($message, "Error");
        wc_add_notice(__($message, "woocommerce"), "error");
        $order_url = $payment_gateway->get_return_url($order);
        wp_redirect($order_url);
        exit();
    } else {
        if( !empty($message) ){
            $order->add_order_note($message, false);
        }
        $order->update_status("wc-failed", $message);
        $woocommerce->cart->empty_cart();
        wc_add_notice($message, "Error");
        wc_add_notice(__($message, "woocommerce"), "error");
        $order_url = $payment_gateway->get_return_url($order);
        wp_redirect($order_url);
        exit();
    }

}
add_action("wp_ajax_nopriv_payomatix_callback_pending_tx", "PayomatixCallbackPendingTransaction");
add_action("wp_ajax_payomatix_callback_pending_tx","PayomatixCallbackPendingTransaction");

add_filter("query_vars", "payomatix_query_vars");
add_action("init", "payomatix_payment_callback_urls");

function payomatix_query_vars($vars)
{
    $vars[] = "order_id";
    $vars[] = "status";
    $vars[] = "response";
    $vars[] = "merchant_ref";
    return $vars;
}

function payomatix_payment_callback_urls()
{
    add_rewrite_rule(
        "^payomatix-callback/(\w)?",
        'index.php?merchant_ref=$matches[1]',
        "top"
    );
}
add_action("parse_request", "payomatix_total_callback");
function payomatix_total_callback($wp)
{
    $payomatixTotalCallback = new PayoMatixTotalCallback();
    $payomatixTotalCallback->payomatixCallback($wp);
}

class PayoMatixTotalCallback extends WC_Payment_Gateway
{
    private $order_status;

    public function __construct()
    {
        $this->id = "payomatix_payment";
        $this->method_title = __(
            "PayoMatix Payment",
            "woocommerce-payomatix-gateway"
        );
        $this->method_description = __(
            "Payomatix Payment getway provide direct payment",
            "woocommerce-payomatix-gateway"
        );
        $this->title = __("Payomatix Payment", "woocommerce-payomatix-gateway");
        $this->init_settings();
        $this->order_status = $this->get_option("order_status");
    }

    public function payomatixCallback($wp)
    {
        $valid_actions = ["merchant_ref"];
        if (
            isset($wp->query_vars["merchant_ref"]) &&
            !empty($wp->query_vars["merchant_ref"])
        ) {
        
            $orderId = $wp->query_vars["merchant_ref"];
            $status = $wp->query_vars["status"];
            $message = isset($wp->query_vars["reason"])
                ? $wp->query_vars["reason"]
                : "";
            if (empty($message)) {
                $message = isset($wp->query_vars["message"])
                    ? $wp->query_vars["message"]
                    : "";
            }

            if ($status == "success") {
                global $woocommerce;

                // we need it to get any order detailes
                $order = wc_get_order($orderId);

                $order->payment_complete();
                $order->reduce_order_stock();
                if( !empty($message) ){
                    $order->add_order_note($message, false);
                }
                $order->update_status($this->order_status, $message);
                $woocommerce->cart->empty_cart();
                wc_add_notice($message, "Success");
                wc_add_notice(__($message, "woocommerce"), "success");
                $order_url = $this->get_return_url($order);
                wp_redirect($order_url);
                exit();
            } elseif ($status == "declined") {
                global $woocommerce;

                // we need it to get any order detailes
                $order = wc_get_order($orderId);

                $order->payment_complete();
                if( !empty($message) ){
                    $order->add_order_note($message, false);
                }
                $order->update_status("wc-failed", $message);
                $woocommerce->cart->empty_cart();
                wc_add_notice($message, "Error");
                wc_add_notice(__($message, "woocommerce"), "error");
                $order_url = $this->get_return_url($order);
                wp_redirect($order_url);
                exit();
            } else {
                global $woocommerce;
                $order = wc_get_order($orderId);
                if( !empty($message) ){
                    $order->add_order_note($message, false);
                }
                $order->update_status("wc-failed", $message);
                $woocommerce->cart->empty_cart();
                wc_add_notice($message, "Error");
                wc_add_notice(__($message, "woocommerce"), "error");
                $order_url = $this->get_return_url($order);
                wp_redirect($order_url);
                exit();
            }
        }
    }

}
